@@ -1,6 +1,7 @@
 /* cursor_type.c - python interface to cursor objects
  *
  * Copyright (C) 2003-2019 Federico Di Gregorio <fog@debian.org>
+ * Copyright (C) 2020 The Psycopg Team
  *
  * This file is part of psycopg.
  *
@@ -317,7 +318,7 @@ _psyco_curs_merge_query_args(cursorObject *self,
 {
     PyObject *fquery;
 
-    /* if PyString_Format() return NULL an error occured: if the error is
+    /* if PyString_Format() return NULL an error occurred: if the error is
        a TypeError we need to check the exception.args[0] string for the
        values:
 
@@ -1827,7 +1828,7 @@ static struct PyMemberDef cursorObject_members[] = {
         "Number of records ``iter(cur)`` must fetch per network roundtrip."},
     {"description", T_OBJECT, OFFSETOF(description), READONLY,
         "Cursor description as defined in DBAPI-2.0."},
-    {"lastrowid", T_LONG, OFFSETOF(lastoid), READONLY,
+    {"lastrowid", T_OID, OFFSETOF(lastoid), READONLY,
         "The ``oid`` of the last row inserted by the cursor."},
     /* DBAPI-2.0 extensions */
     {"rownumber", T_LONG, OFFSETOF(row), READONLY,
