@@ -1,6 +1,7 @@
 /* lobject.h - definition for the psycopg lobject type
  *
- * Copyright (C) 2006-2010 Federico Di Gregorio <fog@debian.org>
+ * Copyright (C) 2006-2019 Federico Di Gregorio <fog@debian.org>
+ * Copyright (C) 2020 The Psycopg Team
  *
  * This file is part of psycopg.
  *
@@ -26,11 +27,8 @@
 #ifndef PSYCOPG_LOBJECT_H
 #define PSYCOPG_LOBJECT_H 1
 
-#include <Python.h>
-#include <libpq-fe.h>
 #include <libpq/libpq-fs.h>
 
-#include "psycopg/config.h"
 #include "psycopg/connection.h"
 
 #ifdef __cplusplus
@@ -45,7 +43,8 @@ typedef struct {
     connectionObject *conn;  /* connection owning the lobject */
     long int mark;           /* copied from conn->mark */
 
-    const char *smode;       /* string mode if lobject was opened */
+    char *smode;             /* string mode if lobject was opened */
+    int mode;                /* numeric version of smode */
 
     int fd;                  /* the file descriptor for file-like ops */
     Oid oid;                 /* the oid for this lobject */
@@ -53,18 +52,19 @@ typedef struct {
 
 /* functions exported from lobject_int.c */
 
-HIDDEN int lobject_open(lobjectObject *self, connectionObject *conn,
-                        Oid oid, int mode, Oid new_oid,
-                        const char *new_file);
-HIDDEN int lobject_unlink(lobjectObject *self);
-HIDDEN int lobject_export(lobjectObject *self, const char *filename);
+RAISES_NEG HIDDEN int lobject_open(lobjectObject *self, connectionObject *conn,
+                                   Oid oid, const char *smode, Oid new_oid,
+                                   const char *new_file);
+RAISES_NEG HIDDEN int lobject_unlink(lobjectObject *self);
+RAISES_NEG HIDDEN int lobject_export(lobjectObject *self, const char *filename);
 
-HIDDEN Py_ssize_t lobject_read(lobjectObject *self, char *buf, size_t len);
-HIDDEN Py_ssize_t lobject_write(lobjectObject *self, const char *buf,
+RAISES_NEG HIDDEN Py_ssize_t lobject_read(lobjectObject *self, char *buf, size_t len);
+RAISES_NEG HIDDEN Py_ssize_t lobject_write(lobjectObject *self, const char *buf,
                                 size_t len);
-HIDDEN int lobject_seek(lobjectObject *self, int pos, int whence);
-HIDDEN int lobject_tell(lobjectObject *self);
-HIDDEN int lobject_close(lobjectObject *self);
+RAISES_NEG HIDDEN Py_ssize_t lobject_seek(lobjectObject *self, Py_ssize_t pos, int whence);
+RAISES_NEG HIDDEN Py_ssize_t lobject_tell(lobjectObject *self);
+RAISES_NEG HIDDEN int lobject_truncate(lobjectObject *self, size_t len);
+RAISES_NEG HIDDEN int lobject_close(lobjectObject *self);
 
 #define lobject_is_closed(self) \
     ((self)->fd < 0 || !(self)->conn || (self)->conn->closed)
@@ -77,18 +77,24 @@ HIDDEN int lobject_close(lobjectObject *self);
     return NULL; }
 
 #define EXC_IF_LOBJ_LEVEL0(self) \
-if (self->conn->isolation_level == 0) {                             \
-    psyco_set_error(ProgrammingError, (PyObject*)self,              \
-        "can't use a lobject outside of transactions", NULL, NULL); \
+if (self->conn->autocommit) {                                       \
+    psyco_set_error(ProgrammingError, NULL,                         \
+        "can't use a lobject outside of transactions");             \
     return NULL;                                                    \
 }
 #define EXC_IF_LOBJ_UNMARKED(self) \
 if (self->conn->mark != self->mark) {                  \
-    psyco_set_error(ProgrammingError, (PyObject*)self, \
-        "lobject isn't valid anymore", NULL, NULL);    \
+    psyco_set_error(ProgrammingError, NULL,            \
+        "lobject isn't valid anymore");                \
     return NULL;                                       \
 }
 
+/* Values for the lobject mode */
+#define LOBJECT_READ  1
+#define LOBJECT_WRITE 2
+#define LOBJECT_BINARY 4
+#define LOBJECT_TEXT 8
+
 #ifdef __cplusplus
 }
 #endif
