@@ -1,6 +1,7 @@
 /* psycopgmodule.c - psycopg module (will import other C classes)
  *
  * Copyright (C) 2003-2019 Federico Di Gregorio <fog@debian.org>
+ * Copyright (C) 2020 The Psycopg Team
  *
  * This file is part of psycopg.
  *
@@ -673,8 +674,8 @@ RAISES_NEG static int
 basic_errors_init(PyObject *module)
 {
     /* the names of the exceptions here reflect the organization of the
-       psycopg2 module and not the fact the the original error objects
-       live in _psycopg */
+       psycopg2 module and not the fact the original error objects live in
+       _psycopg */
 
     int i;
     PyObject *dict = NULL;
