@@ -2,7 +2,8 @@
 
 # test_connection.py - unit test for connection attributes
 #
-# Copyright (C) 2008-2011 James Henstridge  <james@jamesh.id.au>
+# Copyright (C) 2008-2019 James Henstridge  <james@jamesh.id.au>
+# Copyright (C) 2020 The Psycopg Team
 #
 # psycopg2 is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published
@@ -22,24 +23,30 @@
 # FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 # License for more details.
 
-import re
+import gc
 import os
+import re
 import sys
 import time
+import ctypes
+import shutil
+import tempfile
 import threading
 import subprocess as sp
+from collections import deque
 from operator import attrgetter
+from weakref import ref
 
 import psycopg2
-import psycopg2.errorcodes
+import psycopg2.extras
 from psycopg2 import extensions as ext
 
-from testutils import (
-    script_to_py3, unittest, decorate_all_tests, skip_if_no_superuser,
-    skip_before_postgres, skip_after_postgres, skip_before_libpq,
+from .testutils import (
+    PY2, unittest, skip_if_no_superuser, skip_before_postgres,
+    skip_after_postgres, skip_before_libpq, skip_after_libpq,
     ConnectingTestCase, skip_if_tpc_disabled, skip_if_windows, slow)
 
-from testconfig import dsn, dbname
+from .testconfig import dbhost, dsn, dbname
 
 
 class ConnectionTests(ConnectingTestCase):
@@ -87,13 +94,13 @@ class ConnectionTests(ConnectingTestCase):
         conn.autocommit = True
         conn.isolation_level = 'serializable'
         conn.readonly = True
-        if self.conn.server_version >= 90100:
+        if self.conn.info.server_version >= 90100:
             conn.deferrable = False
 
         self.assert_(conn.autocommit)
         self.assertEqual(conn.isolation_level, ext.ISOLATION_LEVEL_SERIALIZABLE)
         self.assert_(conn.readonly is True)
-        if self.conn.server_version >= 90100:
+        if self.conn.info.server_version >= 90100:
             self.assert_(conn.deferrable is False)
 
         conn.reset()
@@ -101,13 +108,13 @@ class ConnectionTests(ConnectingTestCase):
         self.assert_(not conn.autocommit)
         self.assertEqual(conn.isolation_level, ext.ISOLATION_LEVEL_DEFAULT)
         self.assert_(conn.readonly is None)
-        if self.conn.server_version >= 90100:
+        if self.conn.info.server_version >= 90100:
             self.assert_(conn.deferrable is None)
 
     def test_notices(self):
         conn = self.conn
         cur = conn.cursor()
-        if self.conn.server_version >= 90300:
+        if self.conn.info.server_version >= 90300:
             cur.execute("set client_min_messages=debug1")
         cur.execute("create temp table chatty (id serial primary key);")
         self.assertEqual("CREATE TABLE", cur.statusmessage)
@@ -116,7 +123,7 @@ class ConnectionTests(ConnectingTestCase):
     def test_notices_consistent_order(self):
         conn = self.conn
         cur = conn.cursor()
-        if self.conn.server_version >= 90300:
+        if self.conn.info.server_version >= 90300:
             cur.execute("set client_min_messages=debug1")
         cur.execute("""
             create temp table table1 (id serial);
@@ -136,7 +143,7 @@ class ConnectionTests(ConnectingTestCase):
     def test_notices_limited(self):
         conn = self.conn
         cur = conn.cursor()
-        if self.conn.server_version >= 90300:
+        if self.conn.info.server_version >= 90300:
             cur.execute("set client_min_messages=debug1")
         for i in range(0, 100, 10):
             sql = " ".join(["create temp table table%d (id serial);" % j
@@ -148,12 +155,10 @@ class ConnectionTests(ConnectingTestCase):
 
     @slow
     def test_notices_deque(self):
-        from collections import deque
-
         conn = self.conn
         self.conn.notices = deque()
         cur = conn.cursor()
-        if self.conn.server_version >= 90300:
+        if self.conn.info.server_version >= 90300:
             cur.execute("set client_min_messages=debug1")
 
         cur.execute("""
@@ -183,7 +188,7 @@ class ConnectionTests(ConnectingTestCase):
         conn = self.conn
         self.conn.notices = None    # will make an error swallowes ok
         cur = conn.cursor()
-        if self.conn.server_version >= 90300:
+        if self.conn.info.server_version >= 90300:
             cur.execute("set client_min_messages=debug1")
 
         cur.execute("create temp table table1 (id serial);")
@@ -199,7 +204,7 @@ class ConnectionTests(ConnectingTestCase):
 
     def test_tpc_unsupported(self):
         cnn = self.conn
-        if cnn.server_version >= 80100:
+        if cnn.info.server_version >= 80100:
             return self.skipTest("tpc is supported")
 
         self.assertRaises(psycopg2.NotSupportedError,
@@ -254,8 +259,6 @@ class ConnectionTests(ConnectingTestCase):
         conn.close()
 
     def test_weakref(self):
-        from weakref import ref
-        import gc
         conn = psycopg2.connect(dsn)
         w = ref(conn)
         conn.close()
@@ -281,7 +284,6 @@ class ConnectionTests(ConnectingTestCase):
         cur = conn.cursor()
         t1 = threading.Thread(target=committer)
         t1.start()
-        i = 1
         for i in range(1000):
             cur.execute("select %s;", (i,))
             conn.commit()
@@ -294,7 +296,6 @@ class ConnectionTests(ConnectingTestCase):
         self.assert_(not notices, "%d notices raised" % len(notices))
 
     def test_connect_cursor_factory(self):
-        import psycopg2.extras
         conn = self.connect(cursor_factory=psycopg2.extras.DictCursor)
         cur = conn.cursor()
         cur.execute("select 1 as a")
@@ -340,17 +341,83 @@ class ConnectionTests(ConnectingTestCase):
         self.assert_(c.closed, "connection failed so it must be closed")
         self.assert_('foobar' not in c.dsn, "password was not obscured")
 
+    def test_get_native_connection(self):
+        conn = self.connect()
+        capsule = conn.get_native_connection()
+        # we can't do anything else in Python
+        self.assertIsNotNone(capsule)
+
+    def test_pgconn_ptr(self):
+        conn = self.connect()
+        self.assert_(conn.pgconn_ptr is not None)
+
+        try:
+            f = self.libpq.PQserverVersion
+        except AttributeError:
+            pass
+        else:
+            f.argtypes = [ctypes.c_void_p]
+            f.restype = ctypes.c_int
+            ver = f(conn.pgconn_ptr)
+            if ver == 0 and sys.platform == 'darwin':
+                return self.skipTest(
+                    "I don't know why this func returns 0 on OSX")
+
+            self.assertEqual(ver, conn.server_version)
+
+        conn.close()
+        self.assert_(conn.pgconn_ptr is None)
+
+    @slow
+    def test_multiprocess_close(self):
+        dir = tempfile.mkdtemp()
+        try:
+            with open(os.path.join(dir, "mptest.py"), 'w') as f:
+                f.write("""\
+import time
+import psycopg2
+
+def thread():
+    conn = psycopg2.connect(%(dsn)r)
+    curs = conn.cursor()
+    for i in range(10):
+        curs.execute("select 1")
+        time.sleep(0.1)
+
+def process():
+    time.sleep(0.2)
+""" % {'dsn': dsn})
+
+            script = ("""\
+import sys
+sys.path.insert(0, %(dir)r)
+import time
+import threading
+import multiprocessing
+import mptest
+
+t = threading.Thread(target=mptest.thread, name='mythread')
+t.start()
+time.sleep(0.2)
+multiprocessing.Process(target=mptest.process, name='myprocess').start()
+t.join()
+""" % {'dir': dir})
+
+            out = sp.check_output(
+                [sys.executable, '-c', script], stderr=sp.STDOUT)
+            self.assertEqual(out, b'', out)
+        finally:
+            shutil.rmtree(dir, ignore_errors=True)
+
 
 class ParseDsnTestCase(ConnectingTestCase):
     def test_parse_dsn(self):
-        from psycopg2 import ProgrammingError
-
         self.assertEqual(
             ext.parse_dsn('dbname=test user=tester password=secret'),
             dict(user='tester', password='secret', dbname='test'),
             "simple DSN parsed")
 
-        self.assertRaises(ProgrammingError, ext.parse_dsn,
+        self.assertRaises(psycopg2.ProgrammingError, ext.parse_dsn,
                           "dbname=test 2 user=tester password=secret")
 
         self.assertEqual(
@@ -359,18 +426,15 @@ class ParseDsnTestCase(ConnectingTestCase):
             "DSN with quoting parsed")
 
         # Can't really use assertRaisesRegexp() here since we need to
-        # make sure that secret is *not* exposed in the error messgage
-        # (and it also requires python >= 2.7).
+        # make sure that secret is *not* exposed in the error message.
         raised = False
         try:
             # unterminated quote after dbname:
             ext.parse_dsn("dbname='test 2 user=tester password=secret")
-        except ProgrammingError as e:
+        except psycopg2.ProgrammingError as e:
             raised = True
             self.assertTrue(str(e).find('secret') < 0,
                             "DSN was not exposed in error message")
-        except e:
-            self.fail("unexpected error condition: " + repr(e))
         self.assertTrue(raised, "ProgrammingError raised due to invalid DSN")
 
     @skip_before_libpq(9, 2)
@@ -387,14 +451,12 @@ class ParseDsnTestCase(ConnectingTestCase):
             raised = True
             self.assertTrue(str(e).find('secret') < 0,
                             "URI was not exposed in error message")
-        except e:
-            self.fail("unexpected error condition: " + repr(e))
         self.assertTrue(raised, "ProgrammingError raised due to invalid URI")
 
     def test_unicode_value(self):
         snowman = u"\u2603"
         d = ext.parse_dsn('dbname=' + snowman)
-        if sys.version_info[0] < 3:
+        if PY2:
             self.assertEqual(d['dbname'], snowman.encode('utf8'))
         else:
             self.assertEqual(d['dbname'], snowman)
@@ -534,7 +596,7 @@ class IsolationLevelsTestCase(ConnectingTestCase):
             conn.set_isolation_level(level)
 
             # the only values available on prehistoric PG versions
-            if conn.server_version < 80000:
+            if conn.info.server_version < 80000:
                 if level in (
                         ext.ISOLATION_LEVEL_READ_UNCOMMITTED,
                         ext.ISOLATION_LEVEL_REPEATABLE_READ):
@@ -590,35 +652,35 @@ class IsolationLevelsTestCase(ConnectingTestCase):
         cur = conn.cursor()
 
         self.assertEqual(ext.TRANSACTION_STATUS_IDLE,
-            conn.get_transaction_status())
+            conn.info.transaction_status)
         cur.execute("insert into isolevel values (10);")
         self.assertEqual(ext.TRANSACTION_STATUS_INTRANS,
-            conn.get_transaction_status())
+            conn.info.transaction_status)
 
         conn.set_isolation_level(
             psycopg2.extensions.ISOLATION_LEVEL_SERIALIZABLE)
         self.assertEqual(psycopg2.extensions.TRANSACTION_STATUS_IDLE,
-            conn.get_transaction_status())
+            conn.info.transaction_status)
         cur.execute("select count(*) from isolevel;")
         self.assertEqual(0, cur.fetchone()[0])
 
         cur.execute("insert into isolevel values (10);")
         self.assertEqual(psycopg2.extensions.TRANSACTION_STATUS_INTRANS,
-            conn.get_transaction_status())
+            conn.info.transaction_status)
         conn.set_isolation_level(
             psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)
         self.assertEqual(psycopg2.extensions.TRANSACTION_STATUS_IDLE,
-            conn.get_transaction_status())
+            conn.info.transaction_status)
         cur.execute("select count(*) from isolevel;")
         self.assertEqual(0, cur.fetchone()[0])
 
         cur.execute("insert into isolevel values (10);")
         self.assertEqual(psycopg2.extensions.TRANSACTION_STATUS_IDLE,
-            conn.get_transaction_status())
+            conn.info.transaction_status)
         conn.set_isolation_level(
             psycopg2.extensions.ISOLATION_LEVEL_READ_COMMITTED)
         self.assertEqual(psycopg2.extensions.TRANSACTION_STATUS_IDLE,
-            conn.get_transaction_status())
+            conn.info.transaction_status)
         cur.execute("select count(*) from isolevel;")
         self.assertEqual(1, cur.fetchone()[0])
         self.assertEqual(conn.isolation_level,
@@ -714,7 +776,7 @@ class IsolationLevelsTestCase(ConnectingTestCase):
 
         self.conn.isolation_level = ext.ISOLATION_LEVEL_REPEATABLE_READ
         cur.execute("SHOW transaction_isolation;")
-        if self.conn.server_version > 80000:
+        if self.conn.info.server_version > 80000:
             self.assertEqual(self.conn.isolation_level,
                 ext.ISOLATION_LEVEL_REPEATABLE_READ)
             self.assertEqual(cur.fetchone()[0], 'repeatable read')
@@ -733,7 +795,7 @@ class IsolationLevelsTestCase(ConnectingTestCase):
 
         self.conn.isolation_level = ext.ISOLATION_LEVEL_READ_UNCOMMITTED
         cur.execute("SHOW transaction_isolation;")
-        if self.conn.server_version > 80000:
+        if self.conn.info.server_version > 80000:
             self.assertEqual(self.conn.isolation_level,
                 ext.ISOLATION_LEVEL_READ_UNCOMMITTED)
             self.assertEqual(cur.fetchone()[0], 'read uncommitted')
@@ -762,7 +824,7 @@ class IsolationLevelsTestCase(ConnectingTestCase):
 
         self.conn.isolation_level = "repeatable read"
         cur.execute("SHOW transaction_isolation;")
-        if self.conn.server_version > 80000:
+        if self.conn.info.server_version > 80000:
             self.assertEqual(self.conn.isolation_level,
                 ext.ISOLATION_LEVEL_REPEATABLE_READ)
             self.assertEqual(cur.fetchone()[0], 'repeatable read')
@@ -781,7 +843,7 @@ class IsolationLevelsTestCase(ConnectingTestCase):
 
         self.conn.isolation_level = "read uncommitted"
         cur.execute("SHOW transaction_isolation;")
-        if self.conn.server_version > 80000:
+        if self.conn.info.server_version > 80000:
             self.assertEqual(self.conn.isolation_level,
                 ext.ISOLATION_LEVEL_READ_UNCOMMITTED)
             self.assertEqual(cur.fetchone()[0], 'read uncommitted')
@@ -813,6 +875,7 @@ class IsolationLevelsTestCase(ConnectingTestCase):
             self.conn.isolation_level
 
 
+@skip_if_tpc_disabled
 class ConnectionTwoPhaseTests(ConnectingTestCase):
     def setUp(self):
         ConnectingTestCase.setUp(self)
@@ -1109,33 +1172,27 @@ class ConnectionTwoPhaseTests(ConnectingTestCase):
             cnn.tpc_rollback(xid)
 
     def test_xid_construction(self):
-        from psycopg2.extensions import Xid
-
-        x1 = Xid(74, 'foo', 'bar')
+        x1 = ext.Xid(74, 'foo', 'bar')
         self.assertEqual(74, x1.format_id)
         self.assertEqual('foo', x1.gtrid)
         self.assertEqual('bar', x1.bqual)
 
     def test_xid_from_string(self):
-        from psycopg2.extensions import Xid
-
-        x2 = Xid.from_string('42_Z3RyaWQ=_YnF1YWw=')
+        x2 = ext.Xid.from_string('42_Z3RyaWQ=_YnF1YWw=')
         self.assertEqual(42, x2.format_id)
         self.assertEqual('gtrid', x2.gtrid)
         self.assertEqual('bqual', x2.bqual)
 
-        x3 = Xid.from_string('99_xxx_yyy')
+        x3 = ext.Xid.from_string('99_xxx_yyy')
         self.assertEqual(None, x3.format_id)
         self.assertEqual('99_xxx_yyy', x3.gtrid)
         self.assertEqual(None, x3.bqual)
 
     def test_xid_to_string(self):
-        from psycopg2.extensions import Xid
-
-        x1 = Xid.from_string('42_Z3RyaWQ=_YnF1YWw=')
+        x1 = ext.Xid.from_string('42_Z3RyaWQ=_YnF1YWw=')
         self.assertEqual(str(x1), '42_Z3RyaWQ=_YnF1YWw=')
 
-        x2 = Xid.from_string('99_xxx_yyy')
+        x2 = ext.Xid.from_string('99_xxx_yyy')
         self.assertEqual(str(x2), '99_xxx_yyy')
 
     def test_xid_unicode(self):
@@ -1171,8 +1228,7 @@ class ConnectionTwoPhaseTests(ConnectingTestCase):
         self.assertRaises(psycopg2.ProgrammingError, cnn.cancel)
 
     def test_tpc_recover_non_dbapi_connection(self):
-        from psycopg2.extras import RealDictConnection
-        cnn = self.connect(connection_factory=RealDictConnection)
+        cnn = self.connect(connection_factory=psycopg2.extras.RealDictConnection)
         cnn.tpc_begin('dict-connection')
         cnn.tpc_prepare()
         cnn.reset()
@@ -1184,9 +1240,6 @@ class ConnectionTwoPhaseTests(ConnectingTestCase):
         self.assertEqual(None, xid.bqual)
 
 
-decorate_all_tests(ConnectionTwoPhaseTests, skip_if_tpc_disabled)
-
-
 class TransactionControlTests(ConnectingTestCase):
     def test_closed(self):
         self.conn.close()
@@ -1212,7 +1265,7 @@ class TransactionControlTests(ConnectingTestCase):
         self.conn.set_session(
             ext.ISOLATION_LEVEL_REPEATABLE_READ)
         cur.execute("SHOW transaction_isolation;")
-        if self.conn.server_version > 80000:
+        if self.conn.info.server_version > 80000:
             self.assertEqual(cur.fetchone()[0], 'repeatable read')
         else:
             self.assertEqual(cur.fetchone()[0], 'serializable')
@@ -1227,7 +1280,7 @@ class TransactionControlTests(ConnectingTestCase):
         self.conn.set_session(
             isolation_level=ext.ISOLATION_LEVEL_READ_UNCOMMITTED)
         cur.execute("SHOW transaction_isolation;")
-        if self.conn.server_version > 80000:
+        if self.conn.info.server_version > 80000:
             self.assertEqual(cur.fetchone()[0], 'read uncommitted')
         else:
             self.assertEqual(cur.fetchone()[0], 'read committed')
@@ -1242,7 +1295,7 @@ class TransactionControlTests(ConnectingTestCase):
 
         self.conn.set_session("repeatable read")
         cur.execute("SHOW transaction_isolation;")
-        if self.conn.server_version > 80000:
+        if self.conn.info.server_version > 80000:
             self.assertEqual(cur.fetchone()[0], 'repeatable read')
         else:
             self.assertEqual(cur.fetchone()[0], 'serializable')
@@ -1255,7 +1308,7 @@ class TransactionControlTests(ConnectingTestCase):
 
         self.conn.set_session("read uncommitted")
         cur.execute("SHOW transaction_isolation;")
-        if self.conn.server_version > 80000:
+        if self.conn.info.server_version > 80000:
             self.assertEqual(cur.fetchone()[0], 'read uncommitted')
         else:
             self.assertEqual(cur.fetchone()[0], 'read committed')
@@ -1415,6 +1468,86 @@ class TransactionControlTests(ConnectingTestCase):
         self.assertEqual(cur.fetchone()[0], 'on')
 
 
+class TestEncryptPassword(ConnectingTestCase):
+    @skip_before_postgres(10)
+    def test_encrypt_password_post_9_6(self):
+        # MD5 algorithm
+        self.assertEqual(
+            ext.encrypt_password('psycopg2', 'ashesh', self.conn, 'md5'),
+            'md594839d658c28a357126f105b9cb14cfc')
+
+        # keywords
+        self.assertEqual(
+            ext.encrypt_password(
+                password='psycopg2', user='ashesh',
+                scope=self.conn, algorithm='md5'),
+            'md594839d658c28a357126f105b9cb14cfc')
+
+    @skip_before_libpq(10)
+    @skip_before_postgres(10)
+    def test_encrypt_server(self):
+        cur = self.conn.cursor()
+        cur.execute("SHOW password_encryption;")
+        server_encryption_algorithm = cur.fetchone()[0]
+
+        enc_password = ext.encrypt_password(
+            'psycopg2', 'ashesh', self.conn)
+
+        if server_encryption_algorithm == 'md5':
+            self.assertEqual(
+                enc_password, 'md594839d658c28a357126f105b9cb14cfc')
+        elif server_encryption_algorithm == 'scram-sha-256':
+            self.assertEqual(enc_password[:14], 'SCRAM-SHA-256$')
+
+        self.assertEqual(
+            ext.encrypt_password(
+                'psycopg2', 'ashesh', self.conn, 'scram-sha-256'
+            )[:14], 'SCRAM-SHA-256$')
+
+        self.assertRaises(psycopg2.ProgrammingError,
+            ext.encrypt_password, 'psycopg2', 'ashesh', self.conn, 'abc')
+
+    def test_encrypt_md5(self):
+        self.assertEqual(
+            ext.encrypt_password('psycopg2', 'ashesh', algorithm='md5'),
+            'md594839d658c28a357126f105b9cb14cfc')
+
+    @skip_before_libpq(10)
+    def test_encrypt_bad_libpq_10(self):
+        self.assertRaises(psycopg2.ProgrammingError,
+            ext.encrypt_password, 'psycopg2', 'ashesh', self.conn, 'abc')
+
+    @skip_after_libpq(10)
+    def test_encrypt_bad_before_libpq_10(self):
+        self.assertRaises(psycopg2.NotSupportedError,
+            ext.encrypt_password, 'psycopg2', 'ashesh', self.conn, 'abc')
+
+    @skip_before_libpq(10)
+    def test_encrypt_scram(self):
+        self.assert_(
+            ext.encrypt_password(
+                'psycopg2', 'ashesh', self.conn, 'scram-sha-256')
+            .startswith('SCRAM-SHA-256$'))
+
+    @skip_after_libpq(10)
+    def test_encrypt_scram_pre_10(self):
+        self.assertRaises(psycopg2.NotSupportedError,
+            ext.encrypt_password,
+            password='psycopg2', user='ashesh',
+            scope=self.conn, algorithm='scram-sha-256')
+
+    def test_bad_types(self):
+        self.assertRaises(TypeError, ext.encrypt_password)
+        self.assertRaises(TypeError, ext.encrypt_password,
+            'password', 42, self.conn, 'md5')
+        self.assertRaises(TypeError, ext.encrypt_password,
+            42, 'user', self.conn, 'md5')
+        self.assertRaises(TypeError, ext.encrypt_password,
+            42, 'user', 'wat', 'abc')
+        self.assertRaises(TypeError, ext.encrypt_password,
+            'password', 'user', 'wat', 42)
+
+
 class AutocommitTests(ConnectingTestCase):
     def test_closed(self):
         self.conn.close()
@@ -1432,42 +1565,42 @@ class AutocommitTests(ConnectingTestCase):
     def test_default_no_autocommit(self):
         self.assert_(not self.conn.autocommit)
         self.assertEqual(self.conn.status, ext.STATUS_READY)
-        self.assertEqual(self.conn.get_transaction_status(),
+        self.assertEqual(self.conn.info.transaction_status,
             ext.TRANSACTION_STATUS_IDLE)
 
         cur = self.conn.cursor()
         cur.execute('select 1;')
         self.assertEqual(self.conn.status, ext.STATUS_BEGIN)
-        self.assertEqual(self.conn.get_transaction_status(),
+        self.assertEqual(self.conn.info.transaction_status,
             ext.TRANSACTION_STATUS_INTRANS)
 
         self.conn.rollback()
         self.assertEqual(self.conn.status, ext.STATUS_READY)
-        self.assertEqual(self.conn.get_transaction_status(),
+        self.assertEqual(self.conn.info.transaction_status,
             ext.TRANSACTION_STATUS_IDLE)
 
     def test_set_autocommit(self):
         self.conn.autocommit = True
         self.assert_(self.conn.autocommit)
         self.assertEqual(self.conn.status, ext.STATUS_READY)
-        self.assertEqual(self.conn.get_transaction_status(),
+        self.assertEqual(self.conn.info.transaction_status,
             ext.TRANSACTION_STATUS_IDLE)
 
         cur = self.conn.cursor()
         cur.execute('select 1;')
         self.assertEqual(self.conn.status, ext.STATUS_READY)
-        self.assertEqual(self.conn.get_transaction_status(),
+        self.assertEqual(self.conn.info.transaction_status,
             ext.TRANSACTION_STATUS_IDLE)
 
         self.conn.autocommit = False
         self.assert_(not self.conn.autocommit)
         self.assertEqual(self.conn.status, ext.STATUS_READY)
-        self.assertEqual(self.conn.get_transaction_status(),
+        self.assertEqual(self.conn.info.transaction_status,
             ext.TRANSACTION_STATUS_IDLE)
 
         cur.execute('select 1;')
         self.assertEqual(self.conn.status, ext.STATUS_BEGIN)
-        self.assertEqual(self.conn.get_transaction_status(),
+        self.assertEqual(self.conn.info.transaction_status,
             ext.TRANSACTION_STATUS_INTRANS)
 
     def test_set_intrans_error(self):
@@ -1480,24 +1613,24 @@ class AutocommitTests(ConnectingTestCase):
         self.conn.set_session(autocommit=True)
         self.assert_(self.conn.autocommit)
         self.assertEqual(self.conn.status, ext.STATUS_READY)
-        self.assertEqual(self.conn.get_transaction_status(),
+        self.assertEqual(self.conn.info.transaction_status,
             ext.TRANSACTION_STATUS_IDLE)
 
         cur = self.conn.cursor()
         cur.execute('select 1;')
         self.assertEqual(self.conn.status, ext.STATUS_READY)
-        self.assertEqual(self.conn.get_transaction_status(),
+        self.assertEqual(self.conn.info.transaction_status,
             ext.TRANSACTION_STATUS_IDLE)
 
         self.conn.set_session(autocommit=False)
         self.assert_(not self.conn.autocommit)
         self.assertEqual(self.conn.status, ext.STATUS_READY)
-        self.assertEqual(self.conn.get_transaction_status(),
+        self.assertEqual(self.conn.info.transaction_status,
             ext.TRANSACTION_STATUS_IDLE)
 
         cur.execute('select 1;')
         self.assertEqual(self.conn.status, ext.STATUS_BEGIN)
-        self.assertEqual(self.conn.get_transaction_status(),
+        self.assertEqual(self.conn.info.transaction_status,
             ext.TRANSACTION_STATUS_INTRANS)
         self.conn.rollback()
 
@@ -1505,7 +1638,7 @@ class AutocommitTests(ConnectingTestCase):
         self.assert_(self.conn.autocommit)
         cur.execute('select 1;')
         self.assertEqual(self.conn.status, ext.STATUS_READY)
-        self.assertEqual(self.conn.get_transaction_status(),
+        self.assertEqual(self.conn.info.transaction_status,
             ext.TRANSACTION_STATUS_IDLE)
         cur.execute("SHOW transaction_isolation;")
         self.assertEqual(cur.fetchone()[0], 'serializable')
@@ -1603,7 +1736,7 @@ while True:
     cur.execute(%(query)r, ("Hello, world!",))
 """ % {'dsn': dsn, 'query': query})
 
-        proc = sp.Popen([sys.executable, '-c', script_to_py3(script)],
+        proc = sp.Popen([sys.executable, '-c', script],
             stdout=sp.PIPE, stderr=sp.PIPE)
         (out, err) = proc.communicate()
         self.assertNotEqual(proc.returncode, 0)
@@ -1612,8 +1745,168 @@ while True:
         self.assert_(not err, err)
 
 
+class TestConnectionInfo(ConnectingTestCase):
+    def setUp(self):
+        ConnectingTestCase.setUp(self)
+
+        class BrokenConn(psycopg2.extensions.connection):
+            def __init__(self, *args, **kwargs):
+                # don't call superclass
+                pass
+
+        # A "broken" connection
+        self.bconn = self.connect(connection_factory=BrokenConn)
+
+    def test_dbname(self):
+        self.assert_(isinstance(self.conn.info.dbname, str))
+        self.assert_(self.bconn.info.dbname is None)
+
+    def test_user(self):
+        cur = self.conn.cursor()
+        cur.execute("select user")
+        self.assertEqual(self.conn.info.user, cur.fetchone()[0])
+        self.assert_(self.bconn.info.user is None)
+
+    def test_password(self):
+        self.assert_(isinstance(self.conn.info.password, str))
+        self.assert_(self.bconn.info.password is None)
+
+    def test_host(self):
+        expected = dbhost if dbhost else "/"
+        self.assertIn(expected, self.conn.info.host)
+        self.assert_(self.bconn.info.host is None)
+
+    def test_host_readonly(self):
+        with self.assertRaises(AttributeError):
+            self.conn.info.host = 'override'
+
+    def test_port(self):
+        self.assert_(isinstance(self.conn.info.port, int))
+        self.assert_(self.bconn.info.port is None)
+
+    def test_options(self):
+        self.assert_(isinstance(self.conn.info.options, str))
+        self.assert_(self.bconn.info.options is None)
+
+    @skip_before_libpq(9, 3)
+    def test_dsn_parameters(self):
+        d = self.conn.info.dsn_parameters
+        self.assert_(isinstance(d, dict))
+        self.assertEqual(d['dbname'], dbname)  # the only param we can check reliably
+        self.assert_('password' not in d, d)
+
+    def test_status(self):
+        self.assertEqual(self.conn.info.status, 0)
+        self.assertEqual(self.bconn.info.status, 1)
+
+    def test_transaction_status(self):
+        self.assertEqual(self.conn.info.transaction_status, 0)
+        cur = self.conn.cursor()
+        cur.execute("select 1")
+        self.assertEqual(self.conn.info.transaction_status, 2)
+        self.assertEqual(self.bconn.info.transaction_status, 4)
+
+    def test_parameter_status(self):
+        cur = self.conn.cursor()
+        try:
+            cur.execute("show server_version")
+        except psycopg2.DatabaseError:
+            self.assertIsInstance(
+                self.conn.info.parameter_status('server_version'), str)
+        else:
+            self.assertEqual(
+                self.conn.info.parameter_status('server_version'),
+                cur.fetchone()[0])
+
+        self.assertIsNone(self.conn.info.parameter_status('wat'))
+        self.assertIsNone(self.bconn.info.parameter_status('server_version'))
+
+    def test_protocol_version(self):
+        self.assertEqual(self.conn.info.protocol_version, 3)
+        self.assertEqual(self.bconn.info.protocol_version, 0)
+
+    def test_server_version(self):
+        cur = self.conn.cursor()
+        try:
+            cur.execute("show server_version_num")
+        except psycopg2.DatabaseError:
+            self.assert_(isinstance(self.conn.info.server_version, int))
+        else:
+            self.assertEqual(
+                self.conn.info.server_version, int(cur.fetchone()[0]))
+
+        self.assertEqual(self.bconn.info.server_version, 0)
+
+    def test_error_message(self):
+        self.assertIsNone(self.conn.info.error_message)
+        self.assertIsNotNone(self.bconn.info.error_message)
+
+        cur = self.conn.cursor()
+        try:
+            cur.execute("select 1 from nosuchtable")
+        except psycopg2.DatabaseError:
+            pass
+
+        self.assert_('nosuchtable' in self.conn.info.error_message)
+
+    def test_socket(self):
+        self.assert_(self.conn.info.socket >= 0)
+        self.assert_(self.bconn.info.socket < 0)
+
+    def test_backend_pid(self):
+        cur = self.conn.cursor()
+        try:
+            cur.execute("select pg_backend_pid()")
+        except psycopg2.DatabaseError:
+            self.assert_(self.conn.info.backend_pid > 0)
+        else:
+            self.assertEqual(
+                self.conn.info.backend_pid, int(cur.fetchone()[0]))
+
+        self.assert_(self.bconn.info.backend_pid == 0)
+
+    def test_needs_password(self):
+        self.assertIs(self.conn.info.needs_password, False)
+        self.assertIs(self.bconn.info.needs_password, False)
+
+    def test_used_password(self):
+        self.assertIsInstance(self.conn.info.used_password, bool)
+        self.assertIs(self.bconn.info.used_password, False)
+
+    @skip_before_libpq(9, 5)
+    def test_ssl_in_use(self):
+        self.assertIsInstance(self.conn.info.ssl_in_use, bool)
+        self.assertIs(self.bconn.info.ssl_in_use, False)
+
+    @skip_after_libpq(9, 5)
+    def test_ssl_not_supported(self):
+        with self.assertRaises(psycopg2.NotSupportedError):
+            self.conn.info.ssl_in_use
+        with self.assertRaises(psycopg2.NotSupportedError):
+            self.conn.info.ssl_attribute_names
+        with self.assertRaises(psycopg2.NotSupportedError):
+            self.conn.info.ssl_attribute('wat')
+
+    @skip_before_libpq(9, 5)
+    def test_ssl_attribute(self):
+        attribs = self.conn.info.ssl_attribute_names
+        self.assert_(attribs)
+        if self.conn.info.ssl_in_use:
+            for attrib in attribs:
+                self.assertIsInstance(self.conn.info.ssl_attribute(attrib), str)
+        else:
+            for attrib in attribs:
+                self.assertIsNone(self.conn.info.ssl_attribute(attrib))
+
+        self.assertIsNone(self.conn.info.ssl_attribute('wat'))
+
+        for attrib in attribs:
+            self.assertIsNone(self.bconn.info.ssl_attribute(attrib))
+
+
 def test_suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
 
+
 if __name__ == "__main__":
     unittest.main()
