@@ -2,7 +2,8 @@
 
 # test_copy.py - unit test for COPY support
 #
-# Copyright (C) 2010-2011 Daniele Varrazzo  <daniele.varrazzo@gmail.com>
+# Copyright (C) 2010-2019 Daniele Varrazzo  <daniele.varrazzo@gmail.com>
+# Copyright (C) 2020 The Psycopg Team
 #
 # psycopg2 is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published
@@ -22,23 +23,21 @@
 # FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 # License for more details.
 
+import io
 import sys
 import string
-from testutils import unittest, ConnectingTestCase, decorate_all_tests
-from testutils import skip_if_no_iobase, skip_before_postgres
-from cStringIO import StringIO
-from itertools import cycle, izip
+import unittest
+from .testutils import (ConnectingTestCase, skip_before_postgres, slow, StringIO)
+from itertools import cycle
+from subprocess import Popen, PIPE
 
 import psycopg2
 import psycopg2.extensions
-from testutils import skip_copy_if_green
+from .testutils import skip_copy_if_green, PY2, TextIOBase
+from .testconfig import dsn
 
-if sys.version_info[0] < 3:
-    _base = object
-else:
-     from io import TextIOBase as _base
 
-class MinimalRead(_base):
+class MinimalRead(TextIOBase):
     """A file wrapper exposing the minimal interface to copy from."""
     def __init__(self, f):
         self.f = f
@@ -49,7 +48,8 @@ class MinimalRead(_base):
     def readline(self):
         return self.f.readline()
 
-class MinimalWrite(_base):
+
+class MinimalWrite(TextIOBase):
     """A file wrapper exposing the minimal interface to copy to."""
     def __init__(self, f):
         self.f = f
@@ -58,6 +58,7 @@ class MinimalWrite(_base):
         return self.f.write(data)
 
 
+@skip_copy_if_green
 class CopyTests(ConnectingTestCase):
 
     def setUp(self):
@@ -72,26 +73,28 @@ class CopyTests(ConnectingTestCase):
               data text
             )''')
 
+    @slow
     def test_copy_from(self):
         curs = self.conn.cursor()
         try:
-            self._copy_from(curs, nrecs=1024, srec=10*1024, copykw={})
+            self._copy_from(curs, nrecs=1024, srec=10 * 1024, copykw={})
         finally:
             curs.close()
 
+    @slow
     def test_copy_from_insane_size(self):
         # Trying to trigger a "would block" error
         curs = self.conn.cursor()
         try:
-            self._copy_from(curs, nrecs=10*1024, srec=10*1024,
-                copykw={'size': 20*1024*1024})
+            self._copy_from(curs, nrecs=10 * 1024, srec=10 * 1024,
+                copykw={'size': 20 * 1024 * 1024})
         finally:
             curs.close()
 
     def test_copy_from_cols(self):
         curs = self.conn.cursor()
         f = StringIO()
-        for i in xrange(10):
+        for i in range(10):
             f.write("%s\n" % (i,))
 
         f.seek(0)
@@ -103,10 +106,11 @@ class CopyTests(ConnectingTestCase):
     def test_copy_from_cols_err(self):
         curs = self.conn.cursor()
         f = StringIO()
-        for i in xrange(10):
+        for i in range(10):
             f.write("%s\n" % (i,))
 
         f.seek(0)
+
         def cols():
             raise ZeroDivisionError()
             yield 'id'
@@ -114,74 +118,72 @@ class CopyTests(ConnectingTestCase):
         self.assertRaises(ZeroDivisionError,
             curs.copy_from, MinimalRead(f), "tcopy", columns=cols())
 
+    @slow
     def test_copy_to(self):
         curs = self.conn.cursor()
         try:
-            self._copy_from(curs, nrecs=1024, srec=10*1024, copykw={})
-            self._copy_to(curs, srec=10*1024)
+            self._copy_from(curs, nrecs=1024, srec=10 * 1024, copykw={})
+            self._copy_to(curs, srec=10 * 1024)
         finally:
             curs.close()
 
-    @skip_if_no_iobase
     def test_copy_text(self):
         self.conn.set_client_encoding('latin1')
         self._create_temp_table()  # the above call closed the xn
 
-        if sys.version_info[0] < 3:
+        if PY2:
             abin = ''.join(map(chr, range(32, 127) + range(160, 256)))
             about = abin.decode('latin1').replace('\\', '\\\\')
 
         else:
-            abin = bytes(range(32, 127) + range(160, 256)).decode('latin1')
+            abin = bytes(list(range(32, 127))
+                + list(range(160, 256))).decode('latin1')
             about = abin.replace('\\', '\\\\')
 
         curs = self.conn.cursor()
         curs.execute('insert into tcopy values (%s, %s)',
             (42, abin))
 
-        import io
         f = io.StringIO()
         curs.copy_to(f, 'tcopy', columns=('data',))
         f.seek(0)
         self.assertEqual(f.readline().rstrip(), about)
 
-    @skip_if_no_iobase
     def test_copy_bytes(self):
         self.conn.set_client_encoding('latin1')
         self._create_temp_table()  # the above call closed the xn
 
-        if sys.version_info[0] < 3:
+        if PY2:
             abin = ''.join(map(chr, range(32, 127) + range(160, 255)))
             about = abin.replace('\\', '\\\\')
         else:
-            abin = bytes(range(32, 127) + range(160, 255)).decode('latin1')
+            abin = bytes(list(range(32, 127))
+                + list(range(160, 255))).decode('latin1')
             about = abin.replace('\\', '\\\\').encode('latin1')
 
         curs = self.conn.cursor()
         curs.execute('insert into tcopy values (%s, %s)',
             (42, abin))
 
-        import io
         f = io.BytesIO()
         curs.copy_to(f, 'tcopy', columns=('data',))
         f.seek(0)
         self.assertEqual(f.readline().rstrip(), about)
 
-    @skip_if_no_iobase
     def test_copy_expert_textiobase(self):
         self.conn.set_client_encoding('latin1')
         self._create_temp_table()  # the above call closed the xn
 
-        if sys.version_info[0] < 3:
+        if PY2:
             abin = ''.join(map(chr, range(32, 127) + range(160, 256)))
             abin = abin.decode('latin1')
             about = abin.replace('\\', '\\\\')
 
         else:
-            abin = bytes(range(32, 127) + range(160, 256)).decode('latin1')
+            abin = bytes(list(range(32, 127))
+                + list(range(160, 256))).decode('latin1')
             about = abin.replace('\\', '\\\\')
 
-        import io
         f = io.StringIO()
         f.write(about)
         f.seek(0)
@@ -206,9 +208,11 @@ class CopyTests(ConnectingTestCase):
         exp_size = 123
         # hack here to leave file as is, only check size when reading
         real_read = f.read
+
         def read(_size, f=f, exp_size=exp_size):
             self.assertEqual(_size, exp_size)
             return real_read(_size)
+
         f.read = read
         curs.copy_expert('COPY tcopy (data) FROM STDIN', f, size=exp_size)
         curs.execute("select data from tcopy;")
@@ -216,9 +220,9 @@ class CopyTests(ConnectingTestCase):
 
     def _copy_from(self, curs, nrecs, srec, copykw):
         f = StringIO()
-        for i, c in izip(xrange(nrecs), cycle(string.ascii_letters)):
+        for i, c in zip(range(nrecs), cycle(string.ascii_letters)):
             l = c * srec
-            f.write("%s\t%s\n" % (i,l))
+            f.write("%s\t%s\n" % (i, l))
 
         f.seek(0)
         curs.copy_from(MinimalRead(f), "tcopy", **copykw)
@@ -255,24 +259,24 @@ class CopyTests(ConnectingTestCase):
             curs.copy_expert, 'COPY tcopy (data) FROM STDIN', f)
 
     def test_copy_no_column_limit(self):
-        cols = [ "c%050d" % i for i in range(200) ]
+        cols = ["c%050d" % i for i in range(200)]
 
         curs = self.conn.cursor()
         curs.execute('CREATE TEMPORARY TABLE manycols (%s)' % ',\n'.join(
-            [ "%s int" % c for c in cols]))
+            ["%s int" % c for c in cols]))
         curs.execute("INSERT INTO manycols DEFAULT VALUES")
 
         f = StringIO()
-        curs.copy_to(f, "manycols", columns = cols)
+        curs.copy_to(f, "manycols", columns=cols)
         f.seek(0)
         self.assertEqual(f.read().split(), ['\\N'] * len(cols))
 
         f.seek(0)
-        curs.copy_from(f, "manycols", columns = cols)
+        curs.copy_from(f, "manycols", columns=cols)
         curs.execute("select count(*) from manycols;")
         self.assertEqual(curs.fetchone()[0], 2)
 
-    @skip_before_postgres(8, 2) # they don't send the count
+    @skip_before_postgres(8, 2)     # they don't send the count
     def test_copy_rowcount(self):
         curs = self.conn.cursor()
 
@@ -301,12 +305,75 @@ class CopyTests(ConnectingTestCase):
             curs.copy_from, StringIO('aaa\nbbb\nccc\n'), 'tcopy')
         self.assertEqual(curs.rowcount, -1)
 
+    @slow
+    def test_copy_from_segfault(self):
+        # issue #219
+        script = ("""\
+import psycopg2
+conn = psycopg2.connect(%(dsn)r)
+curs = conn.cursor()
+curs.execute("create table copy_segf (id int)")
+try:
+    curs.execute("copy copy_segf from stdin")
+except psycopg2.ProgrammingError:
+    pass
+conn.close()
+""" % {'dsn': dsn})
 
-decorate_all_tests(CopyTests, skip_copy_if_green)
+        proc = Popen([sys.executable, '-c', script])
+        proc.communicate()
+        self.assertEqual(0, proc.returncode)
+
+    @slow
+    def test_copy_to_segfault(self):
+        # issue #219
+        script = ("""\
+import psycopg2
+conn = psycopg2.connect(%(dsn)r)
+curs = conn.cursor()
+curs.execute("create table copy_segf (id int)")
+try:
+    curs.execute("copy copy_segf to stdout")
+except psycopg2.ProgrammingError:
+    pass
+conn.close()
+""" % {'dsn': dsn})
+
+        proc = Popen([sys.executable, '-c', script], stdout=PIPE)
+        proc.communicate()
+        self.assertEqual(0, proc.returncode)
+
+    def test_copy_from_propagate_error(self):
+        class BrokenRead(TextIOBase):
+            def read(self, size):
+                return 1 / 0
+
+            def readline(self):
+                return 1 / 0
+
+        curs = self.conn.cursor()
+        # It seems we cannot do this, but now at least we propagate the error
+        # self.assertRaises(ZeroDivisionError,
+        #     curs.copy_from, BrokenRead(), "tcopy")
+        try:
+            curs.copy_from(BrokenRead(), "tcopy")
+        except Exception as e:
+            self.assert_('ZeroDivisionError' in str(e))
+
+    def test_copy_to_propagate_error(self):
+        class BrokenWrite(TextIOBase):
+            def write(self, data):
+                return 1 / 0
+
+        curs = self.conn.cursor()
+        curs.execute("insert into tcopy values (10, 'hi')")
+        self.assertRaises(ZeroDivisionError,
+            curs.copy_to, BrokenWrite(), "tcopy")
 
 
 def test_suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
 
+
 if __name__ == "__main__":
     unittest.main()
