@@ -2,7 +2,8 @@
 
 # test_dates.py - unit test for dates handling
 #
-# Copyright (C) 2008-2011 James Henstridge  <james@jamesh.id.au>
+# Copyright (C) 2008-2019 James Henstridge  <james@jamesh.id.au>
+# Copyright (C) 2020 The Psycopg Team
 #
 # psycopg2 is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published
@@ -23,9 +24,19 @@
 # License for more details.
 
 import math
+import pickle
+from datetime import date, datetime, time, timedelta
+
 import psycopg2
 from psycopg2.tz import FixedOffsetTimezone, ZERO
-from testutils import unittest, ConnectingTestCase, skip_before_postgres
+import unittest
+from .testutils import ConnectingTestCase, skip_before_postgres
+
+try:
+    from mx.DateTime import Date, Time, DateTime, DateTimeDeltaFrom
+except ImportError:
+    # Tests will be skipped
+    pass
 
 
 def total_seconds(d):
@@ -129,7 +140,6 @@ class DatetimeTests(ConnectingTestCase, CommonDatetimeTestsMixin):
         self.assertEqual(value.microsecond, 123456)
 
     def check_time_tz(self, str_offset, offset):
-        from datetime import time, timedelta
         base = time(13, 30, 29)
         base_str = '13:30:29'
 
@@ -160,7 +170,6 @@ class DatetimeTests(ConnectingTestCase, CommonDatetimeTestsMixin):
         self.check_time_tz("-01:15:59", -60 * (60 + 16))
 
     def check_datetime_tz(self, str_offset, offset):
-        from datetime import datetime, timedelta
         base = datetime(2007, 1, 1, 13, 30, 29)
         base_str = '2007-01-01 13:30:29'
 
@@ -228,25 +237,21 @@ class DatetimeTests(ConnectingTestCase, CommonDatetimeTestsMixin):
         self.assertEqual(str(value), '9999-12-31')
 
     def test_adapt_date(self):
-        from datetime import date
         value = self.execute('select (%s)::date::text',
                              [date(2007, 1, 1)])
         self.assertEqual(value, '2007-01-01')
 
     def test_adapt_time(self):
-        from datetime import time
         value = self.execute('select (%s)::time::text',
                              [time(13, 30, 29)])
         self.assertEqual(value, '13:30:29')
 
     def test_adapt_datetime(self):
-        from datetime import datetime
         value = self.execute('select (%s)::timestamp::text',
                              [datetime(2007, 1, 1, 13, 30, 29)])
         self.assertEqual(value, '2007-01-01 13:30:29')
 
     def test_adapt_timedelta(self):
-        from datetime import timedelta
         value = self.execute('select extract(epoch from (%s)::interval)',
                              [timedelta(days=42, seconds=45296,
                                         microseconds=123456)])
@@ -255,7 +260,6 @@ class DatetimeTests(ConnectingTestCase, CommonDatetimeTestsMixin):
         self.assertEqual(int(round((value - seconds) * 1000000)), 123456)
 
     def test_adapt_negative_timedelta(self):
-        from datetime import timedelta
         value = self.execute('select extract(epoch from (%s)::interval)',
                              [timedelta(days=-42, seconds=45296,
                                         microseconds=123456)])
@@ -274,66 +278,51 @@ class DatetimeTests(ConnectingTestCase, CommonDatetimeTestsMixin):
         self.assertEqual(type(o1[0]), type(o2[0]))
 
     def test_type_roundtrip_date(self):
-        from datetime import date
         self._test_type_roundtrip(date(2010, 5, 3))
 
     def test_type_roundtrip_datetime(self):
-        from datetime import datetime
         dt = self._test_type_roundtrip(datetime(2010, 5, 3, 10, 20, 30))
         self.assertEqual(None, dt.tzinfo)
 
     def test_type_roundtrip_datetimetz(self):
-        from datetime import datetime
-        import psycopg2.tz
-        tz = psycopg2.tz.FixedOffsetTimezone(8 * 60)
+        tz = FixedOffsetTimezone(8 * 60)
         dt1 = datetime(2010, 5, 3, 10, 20, 30, tzinfo=tz)
         dt2 = self._test_type_roundtrip(dt1)
         self.assertNotEqual(None, dt2.tzinfo)
         self.assertEqual(dt1, dt2)
 
     def test_type_roundtrip_time(self):
-        from datetime import time
         tm = self._test_type_roundtrip(time(10, 20, 30))
         self.assertEqual(None, tm.tzinfo)
 
     def test_type_roundtrip_timetz(self):
-        from datetime import time
-        import psycopg2.tz
-        tz = psycopg2.tz.FixedOffsetTimezone(8 * 60)
+        tz = FixedOffsetTimezone(8 * 60)
         tm1 = time(10, 20, 30, tzinfo=tz)
         tm2 = self._test_type_roundtrip(tm1)
         self.assertNotEqual(None, tm2.tzinfo)
         self.assertEqual(tm1, tm2)
 
     def test_type_roundtrip_interval(self):
-        from datetime import timedelta
         self._test_type_roundtrip(timedelta(seconds=30))
 
     def test_type_roundtrip_date_array(self):
-        from datetime import date
         self._test_type_roundtrip_array(date(2010, 5, 3))
 
     def test_type_roundtrip_datetime_array(self):
-        from datetime import datetime
         self._test_type_roundtrip_array(datetime(2010, 5, 3, 10, 20, 30))
 
     def test_type_roundtrip_datetimetz_array(self):
-        from datetime import datetime
         self._test_type_roundtrip_array(
             datetime(2010, 5, 3, 10, 20, 30, tzinfo=FixedOffsetTimezone(0)))
 
     def test_type_roundtrip_time_array(self):
-        from datetime import time
         self._test_type_roundtrip_array(time(10, 20, 30))
 
     def test_type_roundtrip_interval_array(self):
-        from datetime import timedelta
         self._test_type_roundtrip_array(timedelta(seconds=30))
 
     @skip_before_postgres(8, 1)
     def test_time_24(self):
-        from datetime import time
-
         t = self.execute("select '24:00'::time;")
         self.assertEqual(t, time(0, 0))
 
@@ -398,8 +387,6 @@ class DatetimeTests(ConnectingTestCase, CommonDatetimeTestsMixin):
         self.assertRaises(OverflowError, f, '00:00:00.100000000000000000')
 
     def test_adapt_infinity_tz(self):
-        from datetime import datetime
-
         t = self.execute("select 'infinity'::timestamp")
         self.assert_(t.tzinfo is None)
         self.assert_(t > datetime(4000, 1, 1))
@@ -424,7 +411,6 @@ class DatetimeTests(ConnectingTestCase, CommonDatetimeTestsMixin):
                 psycopg2.STRING.values, 'WAT', psycopg2.extensions.INTERVAL),
             cur)
 
-        from datetime import timedelta
         for s, v in [
             ('0', timedelta(0)),
             ('1', timedelta(microseconds=1)),
@@ -437,12 +423,19 @@ class DatetimeTests(ConnectingTestCase, CommonDatetimeTestsMixin):
             r = cur.fetchone()[0]
             self.assertEqual(r, v, "%s -> %s != %s" % (s, r, v))
 
-
-# Only run the datetime tests if psycopg was compiled with support.
-if not hasattr(psycopg2.extensions, 'PYDATETIME'):
-    del DatetimeTests
+    @skip_before_postgres(8, 4)
+    def test_interval_iso_8601_not_supported(self):
+        # We may end up supporting, but no pressure for it
+        cur = self.conn.cursor()
+        cur.execute("set local intervalstyle to iso_8601")
+        cur.execute("select '1 day 2 hours'::interval")
+        self.assertRaises(psycopg2.NotSupportedError, cur.fetchone)
 
 
+@unittest.skipUnless(
+    hasattr(psycopg2._psycopg, 'MXDATETIME'),
+    'Requires mx.DateTime support'
+)
 class mxDateTimeTests(ConnectingTestCase, CommonDatetimeTestsMixin):
     """Tests for the mx.DateTime based date handling in psycopg2."""
 
@@ -501,7 +494,6 @@ class mxDateTimeTests(ConnectingTestCase, CommonDatetimeTestsMixin):
 
     def test_parse_time_timezone(self):
         # Time zone information is ignored.
-        from mx.DateTime import Time
         expected = Time(13, 30, 29)
         self.assertEqual(expected, self.TIME("13:30:29+01", self.curs))
         self.assertEqual(expected, self.TIME("13:30:29-01", self.curs))
@@ -512,7 +504,6 @@ class mxDateTimeTests(ConnectingTestCase, CommonDatetimeTestsMixin):
 
     def test_parse_datetime_timezone(self):
         # Time zone information is ignored.
-        from mx.DateTime import DateTime
         expected = DateTime(2007, 1, 1, 13, 30, 29)
         self.assertEqual(
             expected, self.DATETIME("2007-01-01 13:30:29+01", self.curs))
@@ -536,19 +527,16 @@ class mxDateTimeTests(ConnectingTestCase, CommonDatetimeTestsMixin):
         self.assertEqual(value.second, 5)
 
     def test_adapt_time(self):
-        from mx.DateTime import Time
         value = self.execute('select (%s)::time::text',
                              [Time(13, 30, 29)])
         self.assertEqual(value, '13:30:29')
 
     def test_adapt_datetime(self):
-        from mx.DateTime import DateTime
         value = self.execute('select (%s)::timestamp::text',
                              [DateTime(2007, 1, 1, 13, 30, 29.123456)])
         self.assertEqual(value, '2007-01-01 13:30:29.123456')
 
     def test_adapt_bc_datetime(self):
-        from mx.DateTime import DateTime
         value = self.execute('select (%s)::timestamp::text',
                              [DateTime(-41, 1, 1, 13, 30, 29.123456)])
         # microsecs for BC timestamps look not available in PG < 8.4
@@ -558,7 +546,6 @@ class mxDateTimeTests(ConnectingTestCase, CommonDatetimeTestsMixin):
             '0042-01-01 13:30:29 BC'), value)
 
     def test_adapt_timedelta(self):
-        from mx.DateTime import DateTimeDeltaFrom
         value = self.execute('select extract(epoch from (%s)::interval)',
                              [DateTimeDeltaFrom(days=42,
                                                 seconds=45296.123456)])
@@ -567,7 +554,6 @@ class mxDateTimeTests(ConnectingTestCase, CommonDatetimeTestsMixin):
         self.assertEqual(int(round((value - seconds) * 1000000)), 123456)
 
     def test_adapt_negative_timedelta(self):
-        from mx.DateTime import DateTimeDeltaFrom
         value = self.execute('select extract(epoch from (%s)::interval)',
                              [DateTimeDeltaFrom(days=-42,
                                                 seconds=45296.123456)])
@@ -585,64 +571,45 @@ class mxDateTimeTests(ConnectingTestCase, CommonDatetimeTestsMixin):
         self.assertEqual(type(o1[0]), type(o2[0]))
 
     def test_type_roundtrip_date(self):
-        from mx.DateTime import Date
         self._test_type_roundtrip(Date(2010, 5, 3))
 
     def test_type_roundtrip_datetime(self):
-        from mx.DateTime import DateTime
         self._test_type_roundtrip(DateTime(2010, 5, 3, 10, 20, 30))
 
     def test_type_roundtrip_time(self):
-        from mx.DateTime import Time
         self._test_type_roundtrip(Time(10, 20, 30))
 
     def test_type_roundtrip_interval(self):
-        from mx.DateTime import DateTimeDeltaFrom
         self._test_type_roundtrip(DateTimeDeltaFrom(seconds=30))
 
     def test_type_roundtrip_date_array(self):
-        from mx.DateTime import Date
         self._test_type_roundtrip_array(Date(2010, 5, 3))
 
     def test_type_roundtrip_datetime_array(self):
-        from mx.DateTime import DateTime
         self._test_type_roundtrip_array(DateTime(2010, 5, 3, 10, 20, 30))
 
     def test_type_roundtrip_time_array(self):
-        from mx.DateTime import Time
         self._test_type_roundtrip_array(Time(10, 20, 30))
 
     def test_type_roundtrip_interval_array(self):
-        from mx.DateTime import DateTimeDeltaFrom
         self._test_type_roundtrip_array(DateTimeDeltaFrom(seconds=30))
 
 
-# Only run the mx.DateTime tests if psycopg was compiled with support.
-try:
-    if not hasattr(psycopg2._psycopg, 'MXDATETIME'):
-        del mxDateTimeTests
-except AttributeError:
-    del mxDateTimeTests
-
-
 class FromTicksTestCase(unittest.TestCase):
     # bug "TimestampFromTicks() throws ValueError (2-2.0.14)"
     # reported by Jozsef Szalay on 2010-05-06
     def test_timestamp_value_error_sec_59_99(self):
-        from datetime import datetime
         s = psycopg2.TimestampFromTicks(1273173119.99992)
         self.assertEqual(s.adapted,
             datetime(2010, 5, 6, 14, 11, 59, 999920,
                 tzinfo=FixedOffsetTimezone(-5 * 60)))
 
     def test_date_value_error_sec_59_99(self):
-        from datetime import date
         s = psycopg2.DateFromTicks(1273173119.99992)
         # The returned date is local
         self.assert_(s.adapted in [date(2010, 5, 6), date(2010, 5, 7)])
 
     def test_time_value_error_sec_59_99(self):
-        from datetime import time
         s = psycopg2.TimeFromTicks(1273173119.99992)
         self.assertEqual(s.adapted.replace(hour=0),
             time(0, 11, 59, 999920))
@@ -684,8 +651,6 @@ class FixedOffsetTimezoneTests(unittest.TestCase):
 
     def test_pickle(self):
         # ticket #135
-        import pickle
-
         tz11 = FixedOffsetTimezone(60)
         tz12 = FixedOffsetTimezone(120)
         for proto in [-1, 0, 1, 2]:
@@ -704,5 +669,6 @@ class FixedOffsetTimezoneTests(unittest.TestCase):
 def test_suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
 
+
 if __name__ == "__main__":
     unittest.main()
