@@ -3,6 +3,7 @@
 # extras_dictcursor - test if DictCursor extension class works
 #
 # Copyright (C) 2004-2019 Federico Di Gregorio  <fog@debian.org>
+# Copyright (C) 2020 The Psycopg Team
 #
 # psycopg2 is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published
@@ -14,6 +15,7 @@
 # FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 # License for more details.
 
+import copy
 import time
 import pickle
 import unittest
@@ -157,6 +159,20 @@ class ExtrasDictCursorTests(_DictCursorBase):
         self.assertEqual(r['b'], r1['b'])
         self.assertEqual(r._index, r1._index)
 
+    def test_copy(self):
+        curs = self.conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
+        curs.execute("select 10 as foo, 'hi' as bar")
+        rv = curs.fetchone()
+        self.assertEqual(len(rv), 2)
+
+        rv2 = copy.copy(rv)
+        self.assertEqual(len(rv2), 2)
+        self.assertEqual(len(rv), 2)
+
+        rv3 = copy.deepcopy(rv)
+        self.assertEqual(len(rv3), 2)
+        self.assertEqual(len(rv), 2)
+
     @skip_from_python(3)
     def test_iter_methods_2(self):
         curs = self.conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
@@ -266,6 +282,20 @@ class ExtrasDictCursorRealTests(_DictCursorBase):
         self.assertEqual(r['a'], r1['a'])
         self.assertEqual(r['b'], r1['b'])
 
+    def test_copy(self):
+        curs = self.conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
+        curs.execute("select 10 as foo, 'hi' as bar")
+        rv = curs.fetchone()
+        self.assertEqual(len(rv), 2)
+
+        rv2 = copy.copy(rv)
+        self.assertEqual(len(rv2), 2)
+        self.assertEqual(len(rv), 2)
+
+        rv3 = copy.deepcopy(rv)
+        self.assertEqual(len(rv3), 2)
+        self.assertEqual(len(rv), 2)
+
     def testDictCursorRealWithNamedCursorFetchOne(self):
         self._testWithNamedCursorReal(lambda curs: curs.fetchone())
 
@@ -615,21 +645,31 @@ class NamedTupleCursorTest(ConnectingTestCase):
             self.assertEqual(i + 1, curs.rownumber)
 
     def test_cache(self):
+        NamedTupleCursor._cached_make_nt.cache_clear()
+
         curs = self.conn.cursor()
         curs.execute("select 10 as a, 20 as b")
         r1 = curs.fetchone()
         curs.execute("select 10 as a, 20 as c")
         r2 = curs.fetchone()
+
+        # Get a new cursor to check that the cache works across multiple ones
+        curs = self.conn.cursor()
         curs.execute("select 10 as a, 30 as b")
         r3 = curs.fetchone()
 
         self.assert_(type(r1) is type(r3))
         self.assert_(type(r1) is not type(r2))
 
+        cache_info = NamedTupleCursor._cached_make_nt.cache_info()
+        self.assertEqual(cache_info.hits, 1)
+        self.assertEqual(cache_info.misses, 2)
+        self.assertEqual(cache_info.currsize, 2)
+
     def test_max_cache(self):
         old_func = NamedTupleCursor._cached_make_nt
         NamedTupleCursor._cached_make_nt = \
-            lru_cache(8)(NamedTupleCursor._do_make_nt)
+            lru_cache(8)(NamedTupleCursor._cached_make_nt.__wrapped__)
         try:
             recs = []
             curs = self.conn.cursor()
