@@ -2,7 +2,8 @@
 
 # test_green.py - unit test for async wait callback
 #
-# Copyright (C) 2010-2011 Daniele Varrazzo  <daniele.varrazzo@gmail.com>
+# Copyright (C) 2010-2019 Daniele Varrazzo  <daniele.varrazzo@gmail.com>
+# Copyright (C) 2020 The Psycopg Team
 #
 # psycopg2 is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published
@@ -22,12 +23,16 @@
 # FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 # License for more details.
 
+import select
 import unittest
+import warnings
+
 import psycopg2
 import psycopg2.extensions
 import psycopg2.extras
+from psycopg2.extensions import POLL_OK, POLL_READ, POLL_WRITE
 
-from testutils import ConnectingTestCase, skip_before_postgres, slow
+from .testutils import ConnectingTestCase, skip_before_postgres, slow
 
 
 class ConnectionStub(object):
@@ -55,10 +60,10 @@ class GreenTestCase(ConnectingTestCase):
         ConnectingTestCase.tearDown(self)
         psycopg2.extensions.set_wait_callback(self._cb)
 
-    def set_stub_wait_callback(self, conn):
+    def set_stub_wait_callback(self, conn, cb=None):
         stub = ConnectionStub(conn)
         psycopg2.extensions.set_wait_callback(
-            lambda conn: psycopg2.extras.wait_select(stub))
+            lambda conn: (cb or psycopg2.extras.wait_select)(stub))
         return stub
 
     @slow
@@ -79,7 +84,6 @@ class GreenTestCase(ConnectingTestCase):
         # on high load on linux: probably because the kernel has more
         # buffers ready. A warning may be useful during development,
         # but an error is bad during regression testing.
-        import warnings
         warnings.warn("sending a large query didn't trigger block on write.")
 
     def test_error_in_callback(self):
@@ -117,6 +121,36 @@ class GreenTestCase(ConnectingTestCase):
         self.assertRaises(psycopg2.ProgrammingError,
             cur.execute, "copy (select 1) to stdout")
 
+    @slow
+    @skip_before_postgres(9, 0)
+    def test_non_block_after_notification(self):
+        def wait(conn):
+            while 1:
+                state = conn.poll()
+                if state == POLL_OK:
+                    break
+                elif state == POLL_READ:
+                    select.select([conn.fileno()], [], [], 0.1)
+                elif state == POLL_WRITE:
+                    select.select([], [conn.fileno()], [], 0.1)
+                else:
+                    raise conn.OperationalError("bad state from poll: %s" % state)
+
+        stub = self.set_stub_wait_callback(self.conn, wait)
+        cur = self.conn.cursor()
+        cur.execute("""
+            select 1;
+            do $$
+                begin
+                    raise notice 'hello';
+                end
+            $$ language plpgsql;
+            select pg_sleep(1);
+            """)
+
+        polls = stub.polls.count(POLL_READ)
+        self.assert_(polls > 8, polls)
+
 
 class CallbackErrorTestCase(ConnectingTestCase):
     def setUp(self):
@@ -131,10 +165,7 @@ class CallbackErrorTestCase(ConnectingTestCase):
 
     def crappy_callback(self, conn):
         """green callback failing after `self.to_error` time it is called"""
-        import select
-        from psycopg2.extensions import POLL_OK, POLL_READ, POLL_WRITE
-
-        while 1:
+        while True:
             if self.to_error is not None:
                 self.to_error -= 1
                 if self.to_error <= 0:
@@ -206,5 +237,6 @@ class CallbackErrorTestCase(ConnectingTestCase):
 def test_suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
 
+
 if __name__ == "__main__":
     unittest.main()
