@@ -2,7 +2,8 @@
 
 # test_module.py - unit test for the module interface
 #
-# Copyright (C) 2011 Daniele Varrazzo <daniele.varrazzo@gmail.com>
+# Copyright (C) 2011-2019 Daniele Varrazzo <daniele.varrazzo@gmail.com>
+# Copyright (C) 2020 The Psycopg Team
 #
 # psycopg2 is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published
@@ -22,12 +23,16 @@
 # FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 # License for more details.
 
+import gc
 import os
 import sys
+import pickle
 from subprocess import Popen
+from weakref import ref
 
-from testutils import (unittest, skip_before_python, skip_before_postgres,
-    ConnectingTestCase, skip_copy_if_green, script_to_py3, slow)
+import unittest
+from .testutils import (skip_before_postgres,
+    ConnectingTestCase, skip_copy_if_green, slow, StringIO)
 
 import psycopg2
 
@@ -172,8 +177,8 @@ class ExceptionsTestCase(ConnectingTestCase):
                 'column_name', 'constraint_name', 'context', 'datatype_name',
                 'internal_position', 'internal_query', 'message_detail',
                 'message_hint', 'message_primary', 'schema_name', 'severity',
-                'source_file', 'source_function', 'source_line', 'sqlstate',
-                'statement_position', 'table_name', ]:
+                'severity_nonlocalized', 'source_file', 'source_function',
+                'source_line', 'sqlstate', 'statement_position', 'table_name', ]:
             v = getattr(diag, attr)
             if v is not None:
                 self.assert_(isinstance(v, str))
@@ -189,9 +194,6 @@ class ExceptionsTestCase(ConnectingTestCase):
         self.assertEqual(e.diag.severity, 'ERROR')
 
     def test_diagnostics_life(self):
-        import gc
-        from weakref import ref
-
         def tmp():
             cur = self.conn.cursor()
             try:
@@ -216,7 +218,6 @@ class ExceptionsTestCase(ConnectingTestCase):
 
     @skip_copy_if_green
     def test_diagnostics_copy(self):
-        from StringIO import StringIO
         f = StringIO()
         cur = self.conn.cursor()
         try:
@@ -276,9 +277,16 @@ class ExceptionsTestCase(ConnectingTestCase):
         self.assertEqual(e.diag.constraint_name, "chk_eq1")
         self.assertEqual(e.diag.datatype_name, None)
 
-    @skip_before_python(2, 5)
+    @skip_before_postgres(9, 6)
+    def test_9_6_diagnostics(self):
+        cur = self.conn.cursor()
+        try:
+            cur.execute("select 1 from nosuchtable")
+        except psycopg2.Error as exc:
+            e = exc
+        self.assertEqual(e.diag.severity_nonlocalized, 'ERROR')
+
     def test_pickle(self):
-        import pickle
         cur = self.conn.cursor()
         try:
             cur.execute("select * from nonexist")
@@ -291,10 +299,8 @@ class ExceptionsTestCase(ConnectingTestCase):
         self.assertEqual(e.pgcode, e1.pgcode)
         self.assert_(e1.cursor is None)
 
-    @skip_before_python(2, 5)
     def test_pickle_connection_error(self):
         # segfaults on psycopg 2.5.1 - see ticket #170
-        import pickle
         try:
             psycopg2.connect('dbname=nosuchdatabasemate')
         except psycopg2.Error as exc:
@@ -324,7 +330,7 @@ sys.path.insert(0, %r)
 import _psycopg
 """ % (pardir, pkgdir))
 
-        proc = Popen([sys.executable, '-c', script_to_py3(script)])
+        proc = Popen([sys.executable, '-c', script])
         proc.communicate()
         self.assertEqual(0, proc.returncode)
 
@@ -341,5 +347,6 @@ class TestVersionDiscovery(unittest.TestCase):
 def test_suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
 
+
 if __name__ == "__main__":
     unittest.main()
