@@ -2,7 +2,8 @@
 
 # test_transaction - unit test on transaction behaviour
 #
-# Copyright (C) 2007-2011 Federico Di Gregorio  <fog@debian.org>
+# Copyright (C) 2007-2019 Federico Di Gregorio  <fog@debian.org>
+# Copyright (C) 2020 The Psycopg Team
 #
 # psycopg2 is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published
@@ -23,17 +24,18 @@
 # License for more details.
 
 import threading
-from testutils import unittest, skip_before_postgres
+import unittest
+from .testutils import ConnectingTestCase, skip_before_postgres, slow
 
 import psycopg2
 from psycopg2.extensions import (
     ISOLATION_LEVEL_SERIALIZABLE, STATUS_BEGIN, STATUS_READY)
-from testconfig import dsn
 
-class TransactionTests(unittest.TestCase):
+
+class TransactionTests(ConnectingTestCase):
 
     def setUp(self):
-        self.conn = psycopg2.connect(dsn)
+        ConnectingTestCase.setUp(self)
         self.conn.set_isolation_level(ISOLATION_LEVEL_SERIALIZABLE)
         curs = self.conn.cursor()
         curs.execute('''
@@ -51,9 +53,6 @@ class TransactionTests(unittest.TestCase):
         curs.execute('INSERT INTO table2 VALUES (1, 1)')
         self.conn.commit()
 
-    def tearDown(self):
-        self.conn.close()
-
     def test_rollback(self):
         # Test that rollback undoes changes
         curs = self.conn.cursor()
@@ -93,16 +92,17 @@ class TransactionTests(unittest.TestCase):
         self.assertEqual(curs.fetchone()[0], 1)
 
 
-class DeadlockSerializationTests(unittest.TestCase):
+class DeadlockSerializationTests(ConnectingTestCase):
     """Test deadlock and serialization failure errors."""
 
     def connect(self):
-        conn = psycopg2.connect(dsn)
+        conn = ConnectingTestCase.connect(self)
         conn.set_isolation_level(ISOLATION_LEVEL_SERIALIZABLE)
         return conn
 
     def setUp(self):
-        self.conn = self.connect()
+        ConnectingTestCase.setUp(self)
+
         curs = self.conn.cursor()
         # Drop table if it already exists
         try:
@@ -130,8 +130,10 @@ class DeadlockSerializationTests(unittest.TestCase):
         curs.execute("DROP TABLE table1")
         curs.execute("DROP TABLE table2")
         self.conn.commit()
-        self.conn.close()
 
+        ConnectingTestCase.tearDown(self)
+
+    @slow
     def test_deadlock(self):
         self.thread1_error = self.thread2_error = None
         step1 = threading.Event()
@@ -145,10 +147,11 @@ class DeadlockSerializationTests(unittest.TestCase):
                 step1.set()
                 step2.wait()
                 curs.execute("LOCK table2 IN ACCESS EXCLUSIVE MODE")
-            except psycopg2.DatabaseError, exc:
+            except psycopg2.DatabaseError as exc:
                 self.thread1_error = exc
                 step1.set()
             conn.close()
+
         def task2():
             try:
                 conn = self.connect()
@@ -157,7 +160,7 @@ class DeadlockSerializationTests(unittest.TestCase):
                 curs.execute("LOCK table2 IN ACCESS EXCLUSIVE MODE")
                 step2.set()
                 curs.execute("LOCK table1 IN ACCESS EXCLUSIVE MODE")
-            except psycopg2.DatabaseError, exc:
+            except psycopg2.DatabaseError as exc:
                 self.thread2_error = exc
                 step2.set()
             conn.close()
@@ -176,8 +179,9 @@ class DeadlockSerializationTests(unittest.TestCase):
         self.assertFalse(self.thread1_error and self.thread2_error)
         error = self.thread1_error or self.thread2_error
         self.assertTrue(isinstance(
-                error, psycopg2.extensions.TransactionRollbackError))
+            error, psycopg2.extensions.TransactionRollbackError))
 
+    @slow
     def test_serialisation_failure(self):
         self.thread1_error = self.thread2_error = None
         step1 = threading.Event()
@@ -193,10 +197,11 @@ class DeadlockSerializationTests(unittest.TestCase):
                 step2.wait()
                 curs.execute("UPDATE table1 SET name='task1' WHERE id = 1")
                 conn.commit()
-            except psycopg2.DatabaseError, exc:
+            except psycopg2.DatabaseError as exc:
                 self.thread1_error = exc
                 step1.set()
             conn.close()
+
         def task2():
             try:
                 conn = self.connect()
@@ -204,7 +209,7 @@ class DeadlockSerializationTests(unittest.TestCase):
                 step1.wait()
                 curs.execute("UPDATE table1 SET name='task2' WHERE id = 1")
                 conn.commit()
-            except psycopg2.DatabaseError, exc:
+            except psycopg2.DatabaseError as exc:
                 self.thread2_error = exc
             step2.set()
             conn.close()
@@ -223,19 +228,16 @@ class DeadlockSerializationTests(unittest.TestCase):
         self.assertFalse(self.thread1_error and self.thread2_error)
         error = self.thread1_error or self.thread2_error
         self.assertTrue(isinstance(
-                error, psycopg2.extensions.TransactionRollbackError))
+            error, psycopg2.extensions.TransactionRollbackError))
 
 
-class QueryCancellationTests(unittest.TestCase):
+class QueryCancellationTests(ConnectingTestCase):
     """Tests for query cancellation."""
 
     def setUp(self):
-        self.conn = psycopg2.connect(dsn)
+        ConnectingTestCase.setUp(self)
         self.conn.set_isolation_level(ISOLATION_LEVEL_SERIALIZABLE)
 
-    def tearDown(self):
-        self.conn.close()
-
     @skip_before_postgres(8, 2)
     def test_statement_timeout(self):
         curs = self.conn.cursor()
@@ -248,5 +250,6 @@ class QueryCancellationTests(unittest.TestCase):
 def test_suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
 
+
 if __name__ == "__main__":
     unittest.main()
